/*****************************************************
 * 汎用データロガーの応用事例 : UART転送速度限界調査
 *
 * PSoC での AD変換最大サンプリング周波数は 1M [sample/sec] です。
 * しかし、UART通信の最大通信速度は 921600 [bit/sec] です。AD変換で
 * 取得したデータを PC に転送する場合は、このPC への通信速度がネックになり
 * ます。ここでは、実力どの程度の転送が出来るのか、実験してみます。
 *
 * PSoC側では、1ch 16bit のデータサンプリングを行います。PSoC からPC への
 * データ転送は、リトルエンディアン方式のバイナリ転送を行います。ASCII文字列化
 * した数値データに比べ、データ転送効率が格段に上がります。
 *
 * 但し、検証目的なので、PC へ送信するデータは、PC 側で検証し易いように、0
 * から連番の数値とします。(AD変換自体は PSoC 側で行いますますが、故意に
 * 破棄します。) もし、UART 転送が間に合わなければ、付録PSoC 基板の緑色
 * LED が光ります。また、PC 側のデータ欠損でも確認出来ます。
 *
 * PC 側では、オシロスコープでの可視化はせず、バイナリデータをファイル保存
 * するだけとします。専属の Python アプリケーションを用意しています。
 *
 * 例)
 * AD変換結果(1サンプル16bit) --> 0x0619, 0x0603, 0x0599, ...
 * バイナリ転送(1byteづつ送信) --> 0x19, 0x06, 0x03, 0x06, 0x99, 0x05, ...
 *          ※カンマは、バイナリイメージの分かりやすい表記を示すものです。データには含まれません。
 *
 * Copyright (C) 2019, CQ Publishing Co.,Ltd., all right reserved.
 *
 * 本ソースコードはフリー・ソフトウェアであり、また無保証です。
 * 用途に制限はありません。あなたの責任の下において、個人的・非営利的な
 * ものから商用製品の開発に及ぶ目的に使用・改変・再配布することができます。
 ****************************************************/

#include <stdio.h>
#include "project.h"
#include "main.h"    /* 自作ヘッダ */

/* AD変換結果の格納変数 */
volatile int16 adc_result[ADC_CHANNEL];
volatile uint16 dummy_data = 0;     /* AD変換結果の代わりに、PCへ送信するダミーデータ */
/* AD変換スタンバイ時0, AD変換完了時 > 0 とするAD変換完了フラグ */
volatile int8  adc_done_flag = 0;


int main(void)
{
    uint8  command;
    uint8  state = STATE_STOP;          /* 現在のステート。ステートは2種類 STATE_START, STATE_STOP */
    
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    ADC_SAR_Seq_1_Start();
    ADC_SAR_Seq_1_IRQ_Enable();
    UART_1_Start();
    Pin_LED_Write(0);   /* BSoC内蔵の緑色 LED 消灯 = データオバーフローで消灯 */
    state = STATE_STOP;
    /* PWM は始動しない。PWM を始動すると、AD変換を開始してしまう。 */
    
    /* 汎用データロガーに加えた、追加コンポーネント(150uA 定電流源)起動 */
    IDAC7_1_Start();
    IDAC7_2_Start();
    
    for(;;)
    {
        /* 1回のAD変換完了に伴う、データ処理
         *
         * AD変換の完了をフラグ(adc_ready_flag>=1) で確認して、多チャンネルAD変換結果を PCへ UART送信する。
         *
         * AD変換は、自動生成API である ADC_SAR_Seq_1_INT.c 内の CY_ISR( ADC_SAR_Seq_1_ISR )
         * 割り込み関数内で実装する。AD変換タイミングは、ハードウェアトリガであり、TopDesign.cysch の AD変換
         * コンポーネントの SOC 端子立ち上がりエッジで割り込みが起動する。 CY_ISR( ADC_SAR_Seq_1_ISR )
         * 割り込み関数が呼び出される。
         *
         * CY_ISR( ADC_SAR_Seq_1_ISR ) 割り込み関数の処理 ----------------------------------
         *      AD変換結果を、グローバル変数 adc_result配列に格納する。最後にグローバル変数フラグ
         *      adc_done_flag を1加算する。
         *
         * main() 関数内での処理 -----------------------------------------------------------
         *      AD変換完了の知らせであるグローバル変数フラグ adc_done_flag が非0 であることを確認し
         *      て、多チャンネルのAD変換結果をCSV文字列形式で UART 送信する。最後にフラグ adc_done_flag
         *      を 0 に戻す。
         *
         * (注意) adc_ready_flag > 1 の時は、main() 関数内のデータ処理が遅く(=AD変換速度が速く)
         *        データサンプリング欠損を示唆している。UART通信速度が十分か、AD変換速度が速すぎないか
         *        要確認のこと。
         */
        if(adc_done_flag >= 1){
            /* dummy_data の代わりに、adc_result[0] とすれば、本来のAD 変換結果が送られる。 */
            UART_1_UartPutChar(dummy_data & 0xFF);
            UART_1_UartPutChar((dummy_data & 0xFF00)>>8);
            
            /* ここまでの処理を経た後に、adc_done_flag > 1 の場合は、処理速度が不足していることを示唆する。 */
            if(adc_done_flag > 1){
                /* AD変換速度が速すぎて、データサンプリング欠損している場合の処理 */
                Pin_LED_Write(1);     /* BSoC内蔵の緑色 LED 点灯 */
            }
            adc_done_flag = 0;        /* 最後にAD変換完了フラグを 0 に戻す */
        }

        /* PCからの命令受信処理 何も受信していなければ command = 0 */
        command = (uint8)(UART_1_UartGetChar() & 0x000000FFl);
        switch(command){
            /* AD変換開始命令(ASCIIコード 'a')を受領した場合 */
            case COMMAND_START:
                if(state == STATE_STOP){
                    state = STATE_START;  /* 現在のステートを STATE_START(AD変換開始) にする。 */
                    PWM_1_Start();        /* AD変換開始は、PWM の開始と等価(PWMからの割り込みでAD変換) */
                    Pin_LED_Write(0);     /* BSoC内蔵の緑色 LED 消灯してAD変換開始 */
                    dummy_data = 0;       /* AD変換結果の代わりに、PCへ送信するダミーデータ */
                }
                break;
            /* AD変換停止命令(ASCIIコード 'z')を受領した場合 */
            case COMMAND_STOP:
                if(state == STATE_START){
                    state = STATE_STOP;   /* 現在のステートを STATE_STOP(AD変換停止) にする。 */
                    PWM_1_Stop();         /* AD変換停止は、PWM の停止と等価。 */
                }
                break;
            default:
                /* Do nothing */
                break;
        }
    }
}
