# -*- coding: utf-8 -*-
"""Thread-safe ring buffer

class RingBuffer:
    Generic ring buffer based on numpy.
class BinaryBuffer:
    Ring buffer with binary data as input.
    This inherits the RingBuffer class.

Dependence
----------
python >= 3.6.0
numpy >= 1.13.3

Licence
----------
MIT

History
----------
2018.08.09    rev0.0    New

Copyright
----------
Copyright (C) 2019, CQ Publishing Co.,Ltd., all right reserved.
"""
import threading
import numpy as np


class RingBuffer():
    """RingBuffer

    Realize a buffer that always keeps the latest data
    while circulating the fixed length buffer.
    It is suitable for the purpose of storing stream data
    and displaying graphs. This library is thread-safe,
    allowing control from multiple threads.
    Input / Output data conform to the numpy library.

    Example
    ----------
    import numpy as np
    from ringbuffer import RingBuffer

    # 1000points, 2ch, data type is float32
    # A numpy array with a size of (1000, 2) is prepared.
    ring = RingBuffer(1000, 2, np.float32)

    # Store the 50points, 2ch data.
    #   Channel 0 : [1, 3, 5, 7, ...]
    #   Channel 1 : [2, 4, 6, 8, ...]
    ring.push(np.arange(100).reshape((-1, 2)))

    # Acquire the latest 5points, 2ch data
    now_latest = ring.pop_latest(5)
    # now_latest == np.array([[90,91],[92,93],[94,95],[96,97],[98,99]])
    #   Channel 0 : now_latest[:, 0] = [90,92,94,96,98] (5points per channel)
    #   Channel 1 : now_latest[:, 1] = [91,93,95,97,99]

    # One dimensional array is also possible to store.
    # Reshaped according to the number of channels.
    #   --> parameter.reshape((-1, channel))
    ring.push(np.array([1000, 1100, 1200, 1400]))

    # Acquisition of data also corresponds to copying to an existing array.
    tmp_array = np.empty((3, 2), dtype=np.float32)
    ring.copy_latest(tmp_array)
    print(tmp_array)
    # [[98, 99], [1000, 1100], [1200, 1400]]]

    # You can store as much data as you want.
    # However, only the latest 1000points, 2ch of data are retained.
    """
    VERSION = '0.0'

    def __init__(self, length, ch, data_type=np.float64):
        """Initialize

        Parameters
        ----------
        length :
            Buffer points per channel
        ch :
            Number of channels
        data_type :
            Data type of ring buffer (ex. numpy.int64 ,numpy.float32, etc...)

        Raises
        ----------
        ValueError :
            Initialization failed
        """
        if(ch < 1 or length < 1):
            raise(ValueError('RingBuffer.__init__ parameter'))
        self._ch = ch
        self._length = length
        self._data_type = data_type
        self._buffer = np.zeros((length, ch), dtype=data_type)
        self._anchor = 0
        self._bookmark = 0
        self._same_mark = True
        self._overflow = False
        self._lock = threading.Lock()

    def __str__(self):
        return('<class RingBuffer shape:(%d, %d)>' % (self._length, self._ch))

    @property
    def shape(self):
        """The shape of the ring buffer

        Returns
        ----------
        Tupple : (data points per channel, Number of channels)
        """
        return(self._length, self._ch)

    @property
    def data_type(self):
        """The data type of the ring buffer

        Returns
        ----------
        Data type (ex. numpy.int64 ,numpy.float32, etc...)
        """
        return(self._data_type)

    @property
    def itemsize(self):
        """Number of data bytes per element

        Returns
        ----------
        int (ex. If the data type is numpy.float32, 4 will be returned.)
        """
        return(self._buffer.itemsize)

    def clear(self):
        """Clear the ring buffer

        Initialize all buffer elements to 0. Also clear bookmarks.
        """
        with self._lock:
            self._buffer.fill(0)
            self._anchor = 0
            self._bookmark = 0
            self._same_mark = True
            self._overflow = False

    def clear_bookmark(self):
        """Clear the bookmark

        Clear only bookmarks. Move the bookmark to the latest data
        update point.
        """
        with self._lock:
            self._bookmark = self._anchor
            self._same_mark = True
            self._overflow = False

    def is_overflow_bookmark(self):
        """Confirm bookmark validity

        By updating the data, confirm whether the bookmark is invalidated.

        When you retrieve the data after the bookmark with
        the pop_bookmark () function, move the bookmark to the
        unacquired point.
        As described above, bookmarks are suitable for use in
        preserving stream data.

        However, when data is pushed and updated after the bookmark point,
        the bookmark becomes meaningless.

        Returns
        ----------
        True  : Bookmark invalid
        False : Bookmarks are valid
        """
        return(self._overflow)

    def push(self, data):
        """Store the data in the ring buffer

        Parameters
        ----------
        There are the following three methods.

        1) One dimension numpy.array
            Reshaped according to the number of channels.
            --> parameter.reshape((-1, channel))

        2) Two dimension numpy.array
            The shape is (push points per channel, Number of channels).
            Since all channels are updated, updating of some channels
            is impossible.

        3) list, tupple, etc...
            Objects convertible by numpy.array(). List, tupple, etc...
            The dimension after the transformation need only be
            in conformity with the above 1) 2).

        Returns
        ----------
        int : Number of points per channel successfully stored.
        """
        with self._lock:
            try:
                if(not isinstance(data, np.ndarray)):
                    data = np.array(data, dtype=self._data_type)

                data_shape = data.shape
                if(len(data_shape) == 1):
                    tmp_data = data.reshape((-1, self._ch))
                elif(len(data_shape) == 2 and
                     data_shape[1] == self._ch):
                    tmp_data = data
                else:
                    return(0)

                push_length = tmp_data.shape[0]
                if(push_length == 0):
                    return(0)
                elif(push_length > self._length):
                    self._anchor = 0
                    self._overflow = True
                    self._buffer[:, :] = tmp_data[-self._length:, :]
                    return(self._length)
                elif(push_length <= self._length-self._anchor):
                    self._buffer[self._anchor: self._anchor+push_length,
                                 :] = tmp_data[:, :]
                    if(self._same_mark):
                        self._same_mark = False
                    elif(self._anchor+push_length > self._bookmark and
                         self._anchor <= self._bookmark):
                            self._overflow = True
                    self._anchor += push_length
                    if(self._anchor >= self._length):
                        self._anchor = 0
                    return(push_length)
                else:
                    first_length = self._length-self._anchor
                    second_length = push_length-first_length
                    self._buffer[self._anchor:, :] =\
                        tmp_data[: first_length, :]
                    self._buffer[: second_length, :] =\
                        tmp_data[first_length:, :]
                    if(self._same_mark):
                        self._same_mark = False
                    elif(self._anchor <= self._bookmark):
                        self._overflow = True
                    elif(second_length > self._bookmark):
                        self._overflow = True
                    self._anchor = second_length
                    return(push_length)
            except:
                return(0)

    def pop_latest(self, length=0):
        """Acquire the latest data

        Acquire the numpy.array at the end of the latest data.
        The acquisition length is limited to the size of the ring buffer.

        parameters
        ----------
        length (int) : Number of points per channel. 0 means all elements.

        Returns
        ----------
        numpy.array
        """
        with self._lock:
            if(length <= 0 or length > self._length):
                pop_length = self._length
            else:
                pop_length = length

            if(pop_length <= self._anchor):
                return(self._buffer[self._anchor-pop_length: self._anchor,
                                    :].copy())
            else:
                first_length = self._anchor
                second_length = pop_length-self._anchor
                if(first_length > 0):
                    return(np.vstack((self._buffer[-second_length:, :],
                                      self._buffer[: first_length, :])))
                else:
                    return(self._buffer[-second_length:, :].copy())

    def copy_latest(self, array):
        """Copy the latest data

        Copy the array at the end of the latest data to
        the array of arguments.Copy the array size of the
        argument as full.
        However, if it is larger than the size of the ring buffer,
        it is copied up to the size of the ring buffer.

        parameters
        ----------
        numpy.array :
            The shape must be the shape
            (Acquisition points per channel, Number of channels)

        Returns
        ----------
        int : Copyed points per channel
        """
        if(not isinstance(array, np.ndarray)):
            return(0)
        with self._lock:
            array_shape = array.shape
            if(len(array_shape) == 2 and array_shape[1] == self._ch and
               array_shape[0] > 0):
                if(array_shape[0] > self._length):
                    copy_length = self._length
                else:
                    copy_length = array_shape[0]

                if(copy_length <= self._anchor):
                    array[:copy_length, :] =\
                        self._buffer[self._anchor-copy_length: self._anchor, :]
                    return(copy_length)
                else:
                    first_length = self._anchor
                    second_length = copy_length-self._anchor
                    array[: second_length, :] =\
                        self._buffer[-second_length:, :]
                    if(first_length > 0):
                        array[second_length: copy_length, :] =\
                            self._buffer[: first_length, :]
                    return(copy_length)
            else:
                return(0)

    def pop_bookmark(self, length=0):
        """Acquire the data from bookmark

        Acquire data from bookmarks and move bookmarks to unread.
        When a bookmark is invalid, it can not be acquired.
        If the data is updated beyond the bookmark,
        the bookmark will be invalid.

        Confirm bookmark validity : self.is_overflow_bookmark()
        Clear bookmark and activate: self.clear_bookmark(), self.clear()

        parameters
        ----------
        length (int) :
            Number of max points per channel. It is limited by the length
            from the bookmark point to the latest update point.
            0 means all elements.

        Returns
        ----------
        numpy.array : Valid acquisition data
        None : Either invalid bookmark or no value to Acquire
        """
        with self._lock:
            if(self._anchor == self._bookmark):
                if(self._same_mark):
                    max_length = 0
                else:
                    max_length = self._length
            elif(self._anchor > self._bookmark):
                max_length = self._anchor - self._bookmark
            else:
                max_length = self._length-self._bookmark+self._anchor

            if(length <= 0 or length > max_length):
                pop_length = max_length
            else:
                pop_length = length

            if(self._overflow):
                return(None)
            elif(self._same_mark):
                return(None)
            elif(self._length-self._bookmark >= pop_length):
                oldmark = self._bookmark
                self._bookmark += pop_length
                if(self._bookmark >= self._length):
                    self._bookmark = 0
                if(self._bookmark == self._anchor):
                    self._same_mark = True
                return(self._buffer[oldmark: oldmark+pop_length, :].copy())
            else:
                first_length = self._length-self._bookmark
                second_length = pop_length-first_length
                self._bookmark = second_length
                if(self._bookmark == self._anchor):
                    self._same_mark = True
                if(first_length > 0):
                    return(np.vstack((self._buffer[self._bookmark:, :],
                                      self._buffer[: second_length, :])))
                else:
                    return(self._buffer[: second_length, :].copy())

    def copy_bookmark(self, array):
        """Copy the data from bookmark

        Copy data from bookmark and move bookmarks to unread.
        When a bookmark is invalid, it can not be acquired.

        Copy the array size of the argument as full.
        However, if it is larger than the size of the ring buffer,
        it is copied up to the size of the ring buffer.

        parameters
        ----------
        numpy.array :
            The shape must be the shape
            (Acquisition points per channel, Number of channels)

        Returns
        ----------
        int : Copyed points per channel
        """
        if(not isinstance(array, np.ndarray)):
            return(0)
        with self._lock:
            array_shape = array.shape
            if(len(array_shape) == 2 and array_shape[1] == self._ch and
               array_shape[0] > 0):
                if(self._anchor == self._bookmark):
                    if(self._same_mark):
                        max_length = 0
                    else:
                        max_length = self._length
                elif(self._anchor > self._bookmark):
                    max_length = self._anchor - self._bookmark
                else:
                    max_length = self._length-self._bookmark+self._anchor

                if(array_shape[0] > max_length):
                    pop_length = max_length
                else:
                    pop_length = array_shape[0]

                if(self._overflow):
                    return(0)
                elif(self._same_mark):
                    return(0)
                elif(self._length-self._bookmark >= pop_length):
                    oldmark = self._bookmark
                    self._bookmark += pop_length
                    if(self._bookmark >= self._length):
                        self._bookmark = 0
                    if(self._bookmark == self._anchor):
                        self._same_mark = True
                    array[: pop_length, :] =\
                        self._buffer[oldmark: oldmark+pop_length, :]
                    return(pop_length)
                else:
                    first_length = self._length-self._bookmark
                    second_length = pop_length-first_length
                    self._bookmark = second_length
                    if(self._bookmark == self._anchor):
                        self._same_mark = True
                    if(first_length > 0):
                        array[: first_length, :] =\
                            self._buffer[self._bookmark:, :]
                    array[first_length: pop_length] =\
                        self._buffer[: second_length, :]
                    return(pop_length)
            else:
                return(0)

    def pop_raw(self, length=0):
        """Acquire the data from the ring buffer

        Get data from the first element of the ring buffer itself.
        The acquisition length is limited to the size of the ring buffer.

        Parameters
        ----------
        length (int) : Number of points per channel. 0 means all elements.

        Returns
        ----------
        numpy.array
        """
        with self._lock:
            if(length <= 0 or length > self._length):
                pop_length = self._length
            else:
                pop_length = length

            return(self._buffer[: pop_length, :].copy())

    def copy_raw(self, array):
        """Copy the data from the ring buffer

        Copy data from the first element of the ring buffer itself.
        Copy the array size of the argument as full.
        However, if it is larger than the size of the ring buffer,
        it is copied up to the size of the ring buffer.

        parameters
        ----------
        numpy.array :
            The shape must be the shape
            (Acquisition points per channel, Number of channels)

        Returns
        ----------
        int : Copyed points per channel
        """
        if(not isinstance(array, np.ndarray)):
            return(0)
        with self._lock:
            array_shape = array.shape
            if(len(array_shape) == 2 and array_shape[1] == self._ch and
               array_shape[0] > 0):
                if(array_shape[0] > self._length):
                    copy_length = self._length
                else:
                    copy_length = array_shape[0]

                array[: copy_length, :] = self._buffer[: copy_length, :]
                return(copy_length)
            else:
                return(0)
# The end of class RingBuffer -------------------------------


class BinaryBuffer(RingBuffer):
    """BinaryBuffer

    Decodes the binary data and stores it in the ring type buffer.
    Multiply the decoded numerical value with the linear correction
    of the proportional constant and the offset.This library is thread-safe.

    The multi-byte data shall be in little-endian format.
    In the case of signed format, it is assumed to be in two complement form.
    (ex. binary_type = numpy.int16)

    Example
    ----------
    import numpy as np
    from ringbuffer import BinaryBuffer

    # 1000points, 2ch
    # binary data type is int16, ring buffer data type is float32
    # gain = 1/1024, offset = 0.001
    # A numpy array with a size of (1000, 2) is prepared.
    ring = BinaryBuffer(1000, 2, gain=1/1024, offset=0.001,
                        binary_type=np.int16, data_type=np.float32)
    # Gains and offsets can be set for each channel.
    # Gain for channel1=0.1, channel2=0.12, channel3=0.11
    ring2 = BinaryBuffer(100, 3,
                         gain=np.array([0.1, 0.12, 0.11]),
                         offset=np.array([0.001, 0.002, 0.001]),
                         binary_type=np.int16, data_type=np.float32)

    # Binary format
    #   b'Ch1-data1 Ch2-data1 Ch1-data2 Ch2-data2 ...'
    #   --> Channel1 : [Ch1-data1, Ch1-data2, Ch1-data3, ...]
    #   --> Channel2 : [Ch2-data1, Ch2-data2, Ch2-data3, ...]
    ring.push(b'\x01\x00\xf0\xff\x02\x00\xff\xff')
    now_latest = ring.pop_latest(2)
    # now_latest = an.array([[1, -16], [2, -1]])
    #    Channe0 : now_latest[:, 0] = [1, 2]
    #    Channel : now_latest[:, 1] = [-16, -1]
    #
    # Be aware that the multi-byte data is little endian.
    """

    def __init__(self, length, ch, gain=1, offset=0,
                 binary_type=np.int16, data_type=np.float64):
        """Initialize

        Parameters
        ----------
        length :
            Buffer points per channel
        ch :
            Number of channels
        gain :
            Correction value to be multipled to the binary value.
                scalar : Correction value common to all channels.
                numpy.array : Correction value for each channel
                              shape must be (1, channel) or (channel,)
        offset :
            Correction value to be added to the multipled binary value.
            scalar or numpy.array
        binary_type :
            Data type of binary value (ex. numpy.int16, etc...)
        data_type :
            Data type of ring buffer (ex. numpy.int64 ,numpy.float32, etc...)

        Raises
        ----------
        ValueError :
            Initialization failed
        """
        super().__init__(length, ch, data_type)
        self._binary_type = binary_type
        self._binary_size = np.empty(1, dtype=binary_type).itemsize
        self._gain = gain
        self._offset = offset

    def __str__(self):
        return('<class BinaryBuffer shape:(%d, %d)>' % self.shape)

    @property
    def gain(self):
        """Get gain value"""
        return(self._gain)

    @gain.setter
    def gain(self, value):
        """Set gain value"""
        with self._lock:
            self._gain = value

    @property
    def offset(self):
        """Get offset value"""
        return(self._offset)

    @offset.setter
    def offset(self, value):
        """Set offset value"""
        with self._lock:
            self._offset = value

    @property
    def binary_type(self):
        """Get data type of binary value"""
        return(self._binary_type)

    @binary_type.setter
    def binary_type(self, value):
        """Set data type of binary value"""
        with self._lock:
            self._binary_type = value
            self._binary_size = np.empty(1, dtype=value).itemsize

    @property
    def unitsize(self):
        """Get minimum byte length that can be stored in the ring buffer

        For example, if the binary type is int32
        and the ringbuffer channel is 4, the minimum size
        when pushing data is (32/8)*4=16 bytes.
        This minimum size is returned.
        The binary data length must be an integral multiple of this length.

        Returns
        ----------
        int
        """
        return(self._ch*self._binary_size)

    def push(self, binary):
        """Push binary stream data

        The decoding method from binary is exemplified.
            (Condition)
                binary_type : numpy.int16
                ringbuffer : 2ch

            (Binary data)
                b'\x01\x02\x03\x04\x05\x06\x07\x08...' (little-endian)


            (Ring buffer)
                [[\x0201, \x0403], [\x0605, \x0807], ...]

        Parameters
        ----------
        binary (bytes or bytearray)

        Returns
        ----------
        int : Number of points per channel successfully stored.
        """
        if(len(binary) >= 1 and
           (isinstance(binary, bytes) or isinstance(binary, bytearray))):
            try:
                with self._lock:
                    calc_data = np.frombuffer(
                        binary, dtype=self._binary_type).\
                        reshape((-1, self._ch))*self._gain+self._offset
                return(super().push(calc_data))
            except:
                return(0)
        else:
            return(0)
# The end of class BinaryBuffer -------------------------------
