# -*- coding: utf-8 -*-
"""UART転送速度限界調査アプリケーション by Python

UART通信で、データ最大転送速度を実験するアプリケーションです。

PSoCからは 1ch / 16bit / リトルエンディアン のバイナリ
データが、断続的に送られてきます。本アプリケーションは、バイナリ
データを受信して、バイナリファイルにそのまま保存します。

受信されるデータは、AD変換結果でなく、検証が容易なダミーデータ
です。1サンプル目は1で、次サンプルから1ずつ増加する連番です。
2047 の次は 0 に戻ります。AD変換結果を漏れなく受信されて
いれば、連番が保たれます。

\x01\x00\x02\x00\x03\x00...\xFF\x07\x00\x00\x01\x00...
-------- +++++++ -------... +++++++ ------- +++++++...
 \x0001   \x0002  \x0003...  \x07FF  \x0000  \x0001...

指定された時間 RUN_TIME [sec] だけ、データ受信を継続します。
最後に、保存されたファイルの内容に連番欠損が無いか確認します。

[Windows10 確認環境]
Python      3.7.0
Numpy       1.15.1
Matplotlib  2.2.3
Pyseiral    3.4

[Raspbian OS 確認環境]
Raspberry Pi ZeroW
Python      3.5.3
Numpy       1.15.4
Matplotlib  3.0.2
Pyseiral    3.2.1

本ソースコードはフリー・ソフトウェアであり、また無保証です。
用途に制限はありません。あなたの責任の下において、個人的・非営利的な
ものから商用製品の開発に及ぶ目的に使用・改変・再配布することができます。
"""
import sys
import time
import serial
from serial.tools import list_ports

# 設定定数 (自由に変更可能です。) *********************************
USB_VID = 0x0403
# UART 通信条件は、マイコンの UART コンポーネントの設定条件に合わせること
# PSoC側の最大UART 転送速度である、921600bps を試行する。
SERIAL_OPTIONS = {'baudrate': 921600,
                  'bytesize': serial.EIGHTBITS,
                  'parity': serial.PARITY_NONE,
                  'stopbits': serial.STOPBITS_ONE}
# データ保存ファイル(必須) PSoC側から送られる、バイナリ形式をそのままに、バイナリ保存。
FILE_NAME = r'speed.bin'
# AD変換継続時間 [sec]
RUN_TIME = 10
# 通信デバイス名(UART 又は USBシリアル変換モジュールのデバイス名) が分かっている場合は、
# DEVICE_NAME に記述する。
# (ex. Windows の USBシリアル変換モジュールの場合 "COM48" 等...COM名
#      Linux の場合 "/dev/ttyAMA0" 等...デバイスファイル名)
# DEVICE_NAME を None にすると、USBシリアル変換モジュール(FTDI社製IC搭載)
# を使っているとみなし、USBデバイスを自動検索する。(Windows, Linux, Mac 共通)
DEVICE_NAME = None

def usb_devicename(vid=0x0403, pid=None):
    """VID, PID から、USBデバイス名を捜索

    引数で指定した Vendor ID, Product ID に合致する
    USB機器を捜索して、あればデバイス名を返す。無ければ、
    None が返る。
    例)  Windows: "COM24", Linux: "/dev/ttyAMA0"

    引数
    ----------
    vid : 捜索したUSB機器の VID(Vendor ID) 番号。※1
    pid : 捜索したUSB機器の PID(Product ID) 番号

    両方(VID,PID) 指定も出来るが、一方を None値としても良い。
    Noneとした方は不問とする。

    戻値
    ----------
    該当するUSB機器のデバイス名

    ※1
    世の中で主流の「USBシリアル変換モジュール」には、FTDI社のICチップが
    使われています。同社の Vendor ID は 0x0403 です。デフォルト
    引数とします。
    """
    # dev_list には、ListPortInfoオブジェクトのリストが返る。
    # ListPortInfo は、接続されている シリアルデバイスの情報を持っている。
    # 接続されているシリアルデバイスの数だけ、リストで返る。
    dev_list = list_ports.comports()
    dev_name = None
    if((vid is not None) and (pid is None)):
        for i in dev_list:
            if(i.vid == vid):
                dev_name = i.device
                break
    elif((vid is None) and (pid is not None)):
        for i in dev_list:
            if(i.pid == pid):
                dev_name = i.device
                break
    elif((vid is not None) and (pid is not None)):
        # dev_list はリストなので、要素を順に for で抽出。
        # i は ListPortInfoオブジェクトになる。
        for i in dev_list:
            # ListPortInfo　オブジェクトは、以下のメンバ変数を有している。
            #    pid (int型) : USB機器である場合、PID(Product ID)番号。
            #    vid (int型)　：　USB機器である場合、VID(Vendor ID)番号。
            #    device (str型) : デバイス名
            if(i.vid == vid and i.pid == pid):
                dev_name = i.device
                break
    return(dev_name)

if(DEVICE_NAME is None):
    # USBシリアル変換モジュール で PSoC と接続
    search_device = usb_devicename(USB_VID)
    if(search_device is None):
        print("Can't open the USB device.")
        sys.exit()
else:
    # ユーザ指定の DEVICE_NAME をデバイス名とする
    search_device = DEVICE_NAME

serial_object = None  # シリアル通信オブジェクト。 None はデバイスクローズ状態。
fp = None             # ファイルオブジェクト。 None はファイルクローズ状態。
try:
    serial_object = serial.Serial(
        search_device, timeout=0, **SERIAL_OPTIONS)
    fp = open(FILE_NAME, mode='wb')
except:
    print("Can't open the USB device or the file.")
    if(serial_object is not None):
        serial_object.close()
    if(fp is not None):
        fp.close()
    sys.exit()

serial_object.flushInput()   # シリアル通信バッファクリア
serial_object.write(b'a')    # マイコンへAD変換開始命令
start_time = time.time()
# 指定時間(RUN_TIME) の間、PSoC側からのデータをファイルにバイナリ保存
while(time.time() - start_time < RUN_TIME):
    if(serial_object.in_waiting > 0):
        fp.write(serial_object.read_all())
serial_object.write(b'z')    # マイコンへAD変換停止命令
time.sleep(0.1)
if(serial_object.in_waiting > 0):    # 最後に残ったデータの取込み
    fp.write(serial_object.read_all())
serial_object.close()        # シリアル通信オブジェクトクローズ
fp.close()

# セーブファイルの検証
with open(FILE_NAME, mode='rb') as fp:
    true_number = 1
    read_bytes = fp.read(2)
    while(read_bytes != b''):
        if(int.from_bytes(read_bytes, 'little') != true_number):
            print("Error: Data is missing. On ", true_number)
            sys.exit()
        else:
            true_number += 1
            if(true_number > 2047):
                true_number = 0
            read_bytes = fp.read(2)

    print("Success: Data is normal.")
